"""
Python module generated from Java source file org.bukkit.event.world.EntitiesUnloadEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Chunk
from org.bukkit.entity import Entity
from org.bukkit.event import HandlerList
from org.bukkit.event.world import *
from typing import Any, Callable, Iterable, Tuple


class EntitiesUnloadEvent(ChunkEvent):
    """
    Called when entities are unloaded.
    
    The provided chunk may or may not be loaded.
    """

    def __init__(self, chunk: "Chunk", entities: list["Entity"]):
        ...


    def getEntities(self) -> list["Entity"]:
        """
        Get the entities which are being unloaded.

        Returns
        - unmodifiable list of unloaded entities.
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.components.EquippableComponent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import NamespacedKey
from org.bukkit import Sound
from org.bukkit import Tag
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.entity import EntityType
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory.meta.components import *
from typing import Any, Callable, Iterable, Tuple


class EquippableComponent(ConfigurationSerializable):
    """
    Represents a component which can turn any item into equippable armor.
    """

    def getSlot(self) -> "EquipmentSlot":
        """
        Gets the slot the item can be equipped to.

        Returns
        - slot
        """
        ...


    def setSlot(self, slot: "EquipmentSlot") -> None:
        """
        Sets the slot the item can be equipped to.

        Arguments
        - slot: new slot
        """
        ...


    def getEquipSound(self) -> "Sound":
        """
        Gets the sound to play when the item is equipped.

        Returns
        - the sound
        """
        ...


    def setEquipSound(self, sound: "Sound") -> None:
        """
        Sets the sound to play when the item is equipped.

        Arguments
        - sound: sound or null for current default
        """
        ...


    def getModel(self) -> "NamespacedKey":
        """
        Gets the key of the model to use when equipped.

        Returns
        - model key
        """
        ...


    def setModel(self, key: "NamespacedKey") -> None:
        """
        Sets the key of the model to use when equipped.

        Arguments
        - key: model key
        """
        ...


    def getCameraOverlay(self) -> "NamespacedKey":
        """
        Gets the key of the camera overlay to use when equipped.

        Returns
        - camera overlay key
        """
        ...


    def setCameraOverlay(self, key: "NamespacedKey") -> None:
        """
        Sets the key of the camera overlay to use when equipped.

        Arguments
        - key: camera overlay key
        """
        ...


    def getAllowedEntities(self) -> Iterable["EntityType"]:
        """
        Gets the entities which can equip this item.

        Returns
        - the entities
        """
        ...


    def setAllowedEntities(self, entities: "EntityType") -> None:
        """
        Sets the entities which can equip this item.

        Arguments
        - entities: the entity types
        """
        ...


    def setAllowedEntities(self, entities: Iterable["EntityType"]) -> None:
        """
        Sets the entities which can equip this item.

        Arguments
        - entities: the entity types
        """
        ...


    def setAllowedEntities(self, tag: "Tag"["EntityType"]) -> None:
        """
        Set the entity types (represented as an entity Tag) which can
        equip this item.

        Arguments
        - tag: the entity tag

        Raises
        - IllegalArgumentException: if the passed `tag` is not an entity
        tag
        """
        ...


    def isDispensable(self) -> bool:
        """
        Gets whether the item can be equipped by a dispenser.

        Returns
        - equippable status
        """
        ...


    def setDispensable(self, dispensable: bool) -> None:
        """
        Sets whether the item can be equipped by a dispenser.

        Arguments
        - dispensable: new equippable status
        """
        ...


    def isSwappable(self) -> bool:
        """
        Gets if the item is swappable by right clicking.

        Returns
        - swappable status
        """
        ...


    def setSwappable(self, swappable: bool) -> None:
        """
        Sets if the item is swappable by right clicking.

        Arguments
        - swappable: new status
        """
        ...


    def isDamageOnHurt(self) -> bool:
        """
        Gets if the item will be damaged when the wearing entity is damaged.

        Returns
        - whether the item will be damaged
        """
        ...


    def setDamageOnHurt(self, damage: bool) -> None:
        """
        Sets if the item will be damaged when the wearing entity is damaged.

        Arguments
        - damage: whether the item will be damaged
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.events.ImplicitTuple

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.events import *
from typing import Any, Callable, Iterable, Tuple


class ImplicitTuple:
    """
    The implicit flag of a scalar event is a pair of boolean values that indicate if the tag may be
    omitted when the scalar is emitted in a plain and non-plain style correspondingly.

    See
    - <a href="http://pyyaml.org/wiki/PyYAMLDocumentation.Events">Events</a>
    """

    def __init__(self, plain: bool, nonplain: bool):
        """
        Create

        Arguments
        - plain: - True when tag can be omitted in plain
        - nonplain: - True when tag can be omitted in non-plain
        """
        ...


    def canOmitTagInPlainScalar(self) -> bool:
        """
        Returns
        - True when tag may be omitted when the scalar is emitted in a plain style.
        """
        ...


    def canOmitTagInNonPlainScalar(self) -> bool:
        """
        Returns
        - True when tag may be omitted when the scalar is emitted in a non-plain style.
        """
        ...


    def bothFalse(self) -> bool:
        """
        getter

        Returns
        - True when both are False
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.collect.HashMultiset

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class HashMultiset(AbstractMapBasedMultiset):
    """
    Multiset implementation backed by a HashMap.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    @staticmethod
    def create() -> "HashMultiset"["E"]:
        """
        Creates a new, empty `HashMultiset` using the default initial capacity.
        """
        ...


    @staticmethod
    def create(distinctElements: int) -> "HashMultiset"["E"]:
        """
        Creates a new, empty `HashMultiset` with the specified expected number of distinct
        elements.

        Arguments
        - distinctElements: the expected number of distinct elements

        Raises
        - IllegalArgumentException: if `distinctElements` is negative
        """
        ...


    @staticmethod
    def create(elements: Iterable["E"]) -> "HashMultiset"["E"]:
        """
        Creates a new `HashMultiset` containing the specified elements.
        
        This implementation is highly efficient when `elements` is itself a Multiset.

        Arguments
        - elements: the elements that the multiset should contain
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Gate

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import Directional
from org.bukkit.block.data import Openable
from org.bukkit.block.data import Powerable
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Gate(Directional, Openable, Powerable):
    """
    'in_wall" indicates if the fence gate is attached to a wall, and if True the
    texture is lowered by a small amount to blend in better.
    """

    def isInWall(self) -> bool:
        """
        Gets the value of the 'in_wall' property.

        Returns
        - the 'in_wall' value
        """
        ...


    def setInWall(self, inWall: bool) -> None:
        """
        Sets the value of the 'in_wall' property.

        Arguments
        - inWall: the new 'in_wall' value
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.view.BeaconView

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory import BeaconInventory
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory.view import *
from org.bukkit.potion import PotionEffectType
from typing import Any, Callable, Iterable, Tuple


class BeaconView(InventoryView):
    """
    An instance of InventoryView which provides extra methods related to
    beacon view data.
    """

    def getTopInventory(self) -> "BeaconInventory":
        ...


    def getTier(self) -> int:
        """
        Gets the tier of the beacon
        
        Beacon tier is deduced by the height of the pyramid the beacon is
        standing on. The level of the beacon is 0 unless the beacon is activated.

        Returns
        - The tier of the beacon
        """
        ...


    def getPrimaryEffect(self) -> "PotionEffectType":
        """
        Gets the primary effect of the beacon.
        
        If the beacon level is high enough where the primary effect can be
        upgraded to level two, e.g. Speed 2. Instead of
        .getSecondaryEffect() being null it .getSecondaryEffect()
        returns the same PotionEffectType as this method.

        Returns
        - The primary effect enabled on the beacon
        """
        ...


    def getSecondaryEffect(self) -> "PotionEffectType":
        """
        Gets the secondary effect of the beacon.
        
        If the beacon level is high enough where the primary effect can be
        upgraded to level two, e.g. Speed 2. The secondary effect will return the
        same effect as .getPrimaryEffect().

        Returns
        - The secondary effect enabled on the beacon
        """
        ...


    def setPrimaryEffect(self, effect: "PotionEffectType") -> None:
        """
        Sets the primary effect of the beacon, or null to clear
        
        The PotionEffectType provided must be one that is already within
        the beacon as a valid option.
        <ol>
        - PotionEffectType.SPEED
        - PotionEffectType.HASTE
        - PotionEffectType.RESISTANCE
        - PotionEffectType.JUMP_BOOST
        - PotionEffectType.STRENGTH
        - PotionEffectType.REGENERATION
        </ol>

        Arguments
        - effect: desired primary effect
        """
        ...


    def setSecondaryEffect(self, effect: "PotionEffectType") -> None:
        """
        Sets the secondary effect on this beacon, or null to clear. Note that
        tier must be &gt;= 4 and a primary effect must be set in order for this
        effect to be active.
        
        The PotionEffectType provided must be one that is already within
        the beacon as a valid option.
        <ol>
        - PotionEffectType.SPEED
        - PotionEffectType.HASTE
        - PotionEffectType.RESISTANCE
        - PotionEffectType.JUMP_BOOST
        - PotionEffectType.STRENGTH
        - PotionEffectType.REGENERATION
        </ol>

        Arguments
        - effect: the desired secondary effect
        """
        ...



"""
Python module generated from Java source file com.google.gson.JsonDeserializer

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from java.lang.reflect import Type
from typing import Any, Callable, Iterable, Tuple


class JsonDeserializer:
    """
    Interface representing a custom deserializer for JSON. You should write a custom deserializer, if
    you are not happy with the default deserialization done by Gson. You will also need to register
    this deserializer through GsonBuilder.registerTypeAdapter(Type, Object).
    
    Let us look at example where defining a deserializer will be useful. The `Id` class
    defined below has two fields: `clazz` and `value`.
    
    ```
    public class Id&lt;T&gt; {
      private final Class&lt;T&gt; clazz;
      private final long value;
      public Id(Class&lt;T&gt; clazz, long value) {
        this.clazz = clazz;
        this.value = value;
      }
      public long getValue() {
        return value;
      }
    }
    ```
    
    The default deserialization of `Id(com.foo.MyObject.class, 20L)` will require the JSON
    string to be `{"clazz":"com.foo.MyObject","value":20}`. Suppose, you already know the
    type of the field that the `Id` will be deserialized into, and hence just want to
    deserialize it from a JSON string `20`. You can achieve that by writing a custom
    deserializer:
    
    ```
    class IdDeserializer implements JsonDeserializer&lt;Id&gt; {
      public Id deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
          throws JsonParseException {
        long idValue = json.getAsJsonPrimitive().getAsLong();
        return new Id((Class) typeOfT, idValue);
      }
    }
    ```
    
    You will also need to register `IdDeserializer` with Gson as follows:
    
    ```
    Gson gson = new GsonBuilder().registerTypeAdapter(Id.class, new IdDeserializer()).create();
    ```
    
    Deserializers should be stateless and thread-safe, otherwise the thread-safety guarantees of
    Gson might not apply.
    
    New applications should prefer TypeAdapter, whose streaming API is more efficient than
    this interface's tree API.
    
    Type `<T>`: type for which the deserializer is being registered. It is possible that a
        deserializer may be asked to deserialize a specific generic type of the T.

    Author(s)
    - Joel Leitch
    """

    def deserialize(self, json: "JsonElement", typeOfT: "Type", context: "JsonDeserializationContext") -> "T":
        """
        Gson invokes this call-back method during deserialization when it encounters a field of the
        specified type.
        
        In the implementation of this call-back method, you should consider invoking JsonDeserializationContext.deserialize(JsonElement, Type) method to create objects for any
        non-trivial field of the returned object. However, you should never invoke it on the same type
        passing `json` since that will cause an infinite loop (Gson will call your call-back
        method again).

        Arguments
        - json: The Json data being deserialized
        - typeOfT: The type of the Object to deserialize to

        Returns
        - a deserialized object of the specified type typeOfT which is a subclass of `T`

        Raises
        - JsonParseException: if json is not in the expected format of `typeOfT`
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockDispenseEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from org.bukkit.inventory import ItemStack
from org.bukkit.util import Vector
from typing import Any, Callable, Iterable, Tuple


class BlockDispenseEvent(BlockEvent, Cancellable):
    """
    Called when an item is dispensed from a block.
    
    If a Block Dispense event is cancelled, the block will not dispense the
    item.
    """

    def __init__(self, block: "Block", dispensed: "ItemStack", velocity: "Vector"):
        ...


    def getItem(self) -> "ItemStack":
        """
        Gets the item that is being dispensed. Modifying the returned item will
        have no effect, you must use .setItem(org.bukkit.inventory.ItemStack) instead.

        Returns
        - An ItemStack for the item being dispensed
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Sets the item being dispensed.

        Arguments
        - item: the item being dispensed
        """
        ...


    def getVelocity(self) -> "Vector":
        """
        Gets the velocity in meters per tick.
        
        Note: Modifying the returned Vector will not change the velocity, you
        must use .setVelocity(org.bukkit.util.Vector) instead.

        Returns
        - A Vector for the dispensed item's velocity
        """
        ...


    def setVelocity(self, vel: "Vector") -> None:
        """
        Sets the velocity of the item being dispensed in meters per tick.

        Arguments
        - vel: the velocity of the item being dispensed
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.io.FileBackedOutputStream

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.io import *
from com.google.errorprone.annotations.concurrent import GuardedBy
from com.google.j2objc.annotations import J2ObjCIncompatible
from java.io import ByteArrayInputStream
from java.io import ByteArrayOutputStream
from java.io import File
from java.io import FileInputStream
from java.io import FileOutputStream
from java.io import IOException
from java.io import InputStream
from java.io import OutputStream
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class FileBackedOutputStream(OutputStream):
    """
    An OutputStream that starts buffering to a byte array, but switches to file buffering
    once the data reaches a configurable size.
    
    When this stream creates a temporary file, it restricts the file's permissions to the current
    user or, in the case of Android, the current app. If that is not possible (as is the case under
    the very old Android Ice Cream Sandwich release), then this stream throws an exception instead of
    creating a file that would be more accessible. (This behavior is new in Guava 32.0.0. Previous
    versions would create a file that is more accessible, as discussed in <a
    href="https://github.com/google/guava/issues/2575">Guava issue 2575</a>. TODO: b/283778848 - Fill
    in CVE number once it's available.)
    
    Temporary files created by this stream may live in the local filesystem until either:
    
    
      - .reset is called (removing the data in this stream and deleting the file), or...
      - this stream (or, more precisely, its .asByteSource view) is finalized during
          garbage collection, <strong>AND</strong> this stream was not constructed with .FileBackedOutputStream(int) the 1-arg constructor or the .FileBackedOutputStream(int, boolean) 2-arg constructor passing `False` in the
          second parameter.
    
    
    This class is thread-safe.

    Author(s)
    - Chris Nokleberg

    Since
    - 1.0
    """

    def __init__(self, fileThreshold: int):
        """
        Creates a new instance that uses the given file threshold, and does not reset the data when the
        ByteSource returned by .asByteSource is finalized.

        Arguments
        - fileThreshold: the number of bytes before the stream should switch to buffering to a file

        Raises
        - IllegalArgumentException: if `fileThreshold` is negative
        """
        ...


    def __init__(self, fileThreshold: int, resetOnFinalize: bool):
        """
        Creates a new instance that uses the given file threshold, and optionally resets the data when
        the ByteSource returned by .asByteSource is finalized.

        Arguments
        - fileThreshold: the number of bytes before the stream should switch to buffering to a file
        - resetOnFinalize: if True, the .reset method will be called when the ByteSource returned by .asByteSource is finalized.

        Raises
        - IllegalArgumentException: if `fileThreshold` is negative
        """
        ...


    def asByteSource(self) -> "ByteSource":
        """
        Returns a readable ByteSource view of the data that has been written to this stream.

        Since
        - 15.0
        """
        ...


    def reset(self) -> None:
        """
        Calls .close if not already closed, and then resets this object back to its initial
        state, for reuse. If data was buffered to a file, it will be deleted.

        Raises
        - IOException: if an I/O error occurred while deleting the file buffer
        """
        ...


    def write(self, b: int) -> None:
        ...


    def write(self, b: list[int]) -> None:
        ...


    def write(self, b: list[int], off: int, len: int) -> None:
        ...


    def close(self) -> None:
        ...


    def flush(self) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.entity.EntityType

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from java.util import Locale
from org.bukkit import Bukkit
from org.bukkit import Keyed
from org.bukkit import Location
from org.bukkit import NamespacedKey
from org.bukkit import Translatable
from org.bukkit import World
from org.bukkit.entity import *
from org.bukkit.entity.boat import AcaciaBoat
from org.bukkit.entity.boat import AcaciaChestBoat
from org.bukkit.entity.boat import BambooChestRaft
from org.bukkit.entity.boat import BambooRaft
from org.bukkit.entity.boat import BirchBoat
from org.bukkit.entity.boat import BirchChestBoat
from org.bukkit.entity.boat import CherryBoat
from org.bukkit.entity.boat import CherryChestBoat
from org.bukkit.entity.boat import DarkOakBoat
from org.bukkit.entity.boat import DarkOakChestBoat
from org.bukkit.entity.boat import JungleBoat
from org.bukkit.entity.boat import JungleChestBoat
from org.bukkit.entity.boat import MangroveBoat
from org.bukkit.entity.boat import MangroveChestBoat
from org.bukkit.entity.boat import OakBoat
from org.bukkit.entity.boat import OakChestBoat
from org.bukkit.entity.boat import PaleOakBoat
from org.bukkit.entity.boat import PaleOakChestBoat
from org.bukkit.entity.boat import SpruceBoat
from org.bukkit.entity.boat import SpruceChestBoat
from org.bukkit.entity.minecart import CommandMinecart
from org.bukkit.entity.minecart import ExplosiveMinecart
from org.bukkit.entity.minecart import HopperMinecart
from org.bukkit.entity.minecart import PoweredMinecart
from org.bukkit.entity.minecart import RideableMinecart
from org.bukkit.entity.minecart import SpawnerMinecart
from org.bukkit.entity.minecart import StorageMinecart
from org.bukkit.inventory import ItemStack
from org.bukkit.potion import PotionEffectType
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class EntityType(Enum):

    ITEM = ("item", Item, 1)
    """
    An item resting on the ground.
    
    Spawn with World.dropItem(Location, ItemStack) or World.dropItemNaturally(Location, ItemStack)
    """
    EXPERIENCE_ORB = ("experience_orb", ExperienceOrb, 2)
    """
    An experience orb.
    """
    AREA_EFFECT_CLOUD = ("area_effect_cloud", AreaEffectCloud, 3)
    """
    See
    - AreaEffectCloud
    """
    ELDER_GUARDIAN = ("elder_guardian", ElderGuardian, 4)
    """
    See
    - ElderGuardian
    """
    WITHER_SKELETON = ("wither_skeleton", WitherSkeleton, 5)
    """
    See
    - WitherSkeleton
    """
    STRAY = ("stray", Stray, 6)
    """
    See
    - Stray
    """
    EGG = ("egg", Egg, 7)
    """
    A flying chicken egg.
    """
    LEASH_KNOT = ("leash_knot", LeashHitch, 8)
    """
    A leash attached to a fencepost.
    """
    PAINTING = ("painting", Painting, 9)
    """
    A painting on a wall.
    """
    ARROW = ("arrow", Arrow, 10)
    """
    An arrow projectile; may get stuck in the ground.
    """
    SNOWBALL = ("snowball", Snowball, 11)
    """
    A flying snowball.
    """
    FIREBALL = ("fireball", LargeFireball, 12)
    """
    A flying large fireball, as thrown by a Ghast for example.
    """
    SMALL_FIREBALL = ("small_fireball", SmallFireball, 13)
    """
    A flying small fireball, such as thrown by a Blaze or player.
    """
    ENDER_PEARL = ("ender_pearl", EnderPearl, 14)
    """
    A flying ender pearl.
    """
    EYE_OF_ENDER = ("eye_of_ender", EnderSignal, 15)
    """
    An ender eye signal.
    """
    POTION = ("potion", ThrownPotion, 16)
    """
    A flying splash potion.
    """
    EXPERIENCE_BOTTLE = ("experience_bottle", ThrownExpBottle, 17)
    """
    A flying experience bottle.
    """
    ITEM_FRAME = ("item_frame", ItemFrame, 18)
    """
    An item frame on a wall.
    """
    WITHER_SKULL = ("wither_skull", WitherSkull, 19)
    """
    A flying wither skull projectile.
    """
    TNT = ("tnt", TNTPrimed, 20)
    """
    Primed TNT that is about to explode.
    """
    FALLING_BLOCK = ("falling_block", FallingBlock, 21)
    """
    A block that is going to or is about to fall.
    """
    FIREWORK_ROCKET = ("firework_rocket", Firework, 22)
    """
    Internal representation of a Firework once it has been launched.
    """
    HUSK = ("husk", Husk, 23)
    """
    See
    - Husk
    """
    SPECTRAL_ARROW = ("spectral_arrow", SpectralArrow, 24)
    """
    Like .ARROW but causes the PotionEffectType.GLOWING effect on all team members.
    """
    SHULKER_BULLET = ("shulker_bullet", ShulkerBullet, 25)
    """
    Bullet fired by .SHULKER.
    """
    DRAGON_FIREBALL = ("dragon_fireball", DragonFireball, 26)
    """
    Like .FIREBALL but with added effects.
    """
    ZOMBIE_VILLAGER = ("zombie_villager", ZombieVillager, 27)
    """
    See
    - ZombieVillager
    """
    SKELETON_HORSE = ("skeleton_horse", SkeletonHorse, 28)
    """
    See
    - SkeletonHorse
    """
    ZOMBIE_HORSE = ("zombie_horse", ZombieHorse, 29)
    """
    See
    - ZombieHorse
    """
    ARMOR_STAND = ("armor_stand", ArmorStand, 30)
    """
    Mechanical entity with an inventory for placing weapons / armor into.
    """
    DONKEY = ("donkey", Donkey, 31)
    """
    See
    - Donkey
    """
    MULE = ("mule", Mule, 32)
    """
    See
    - Mule
    """
    EVOKER_FANGS = ("evoker_fangs", EvokerFangs, 33)
    """
    See
    - EvokerFangs
    """
    EVOKER = ("evoker", Evoker, 34)
    """
    See
    - Evoker
    """
    VEX = ("vex", Vex, 35)
    """
    See
    - Vex
    """
    VINDICATOR = ("vindicator", Vindicator, 36)
    """
    See
    - Vindicator
    """
    ILLUSIONER = ("illusioner", Illusioner, 37)
    """
    See
    - Illusioner
    """
    COMMAND_BLOCK_MINECART = ("command_block_minecart", CommandMinecart, 40)
    """
    See
    - CommandMinecart
    """
    MINECART = ("minecart", RideableMinecart, 42)
    """
    See
    - RideableMinecart
    """
    CHEST_MINECART = ("chest_minecart", StorageMinecart, 43)
    """
    See
    - StorageMinecart
    """
    FURNACE_MINECART = ("furnace_minecart", PoweredMinecart, 44)
    """
    See
    - PoweredMinecart
    """
    TNT_MINECART = ("tnt_minecart", ExplosiveMinecart, 45)
    """
    See
    - ExplosiveMinecart
    """
    HOPPER_MINECART = ("hopper_minecart", HopperMinecart, 46)
    """
    See
    - HopperMinecart
    """
    SPAWNER_MINECART = ("spawner_minecart", SpawnerMinecart, 47)
    """
    See
    - SpawnerMinecart
    """
    CREEPER = ("creeper", Creeper, 50)
    SKELETON = ("skeleton", Skeleton, 51)
    SPIDER = ("spider", Spider, 52)
    GIANT = ("giant", Giant, 53)
    ZOMBIE = ("zombie", Zombie, 54)
    SLIME = ("slime", Slime, 55)
    GHAST = ("ghast", Ghast, 56)
    ZOMBIFIED_PIGLIN = ("zombified_piglin", PigZombie, 57)
    ENDERMAN = ("enderman", Enderman, 58)
    CAVE_SPIDER = ("cave_spider", CaveSpider, 59)
    SILVERFISH = ("silverfish", Silverfish, 60)
    BLAZE = ("blaze", Blaze, 61)
    MAGMA_CUBE = ("magma_cube", MagmaCube, 62)
    ENDER_DRAGON = ("ender_dragon", EnderDragon, 63)
    WITHER = ("wither", Wither, 64)
    BAT = ("bat", Bat, 65)
    WITCH = ("witch", Witch, 66)
    ENDERMITE = ("endermite", Endermite, 67)
    GUARDIAN = ("guardian", Guardian, 68)
    SHULKER = ("shulker", Shulker, 69)
    PIG = ("pig", Pig, 90)
    SHEEP = ("sheep", Sheep, 91)
    COW = ("cow", Cow, 92)
    CHICKEN = ("chicken", Chicken, 93)
    SQUID = ("squid", Squid, 94)
    WOLF = ("wolf", Wolf, 95)
    MOOSHROOM = ("mooshroom", MushroomCow, 96)
    SNOW_GOLEM = ("snow_golem", Snowman, 97)
    OCELOT = ("ocelot", Ocelot, 98)
    IRON_GOLEM = ("iron_golem", IronGolem, 99)
    HORSE = ("horse", Horse, 100)
    RABBIT = ("rabbit", Rabbit, 101)
    POLAR_BEAR = ("polar_bear", PolarBear, 102)
    LLAMA = ("llama", Llama, 103)
    LLAMA_SPIT = ("llama_spit", LlamaSpit, 104)
    PARROT = ("parrot", Parrot, 105)
    VILLAGER = ("villager", Villager, 120)
    END_CRYSTAL = ("end_crystal", EnderCrystal, 200)
    TURTLE = ("turtle", Turtle, -1)
    PHANTOM = ("phantom", Phantom, -1)
    TRIDENT = ("trident", Trident, -1)
    COD = ("cod", Cod, -1)
    SALMON = ("salmon", Salmon, -1)
    PUFFERFISH = ("pufferfish", PufferFish, -1)
    TROPICAL_FISH = ("tropical_fish", TropicalFish, -1)
    DROWNED = ("drowned", Drowned, -1)
    DOLPHIN = ("dolphin", Dolphin, -1)
    CAT = ("cat", Cat, -1)
    PANDA = ("panda", Panda, -1)
    PILLAGER = ("pillager", Pillager, -1)
    RAVAGER = ("ravager", Ravager, -1)
    TRADER_LLAMA = ("trader_llama", TraderLlama, -1)
    WANDERING_TRADER = ("wandering_trader", WanderingTrader, -1)
    FOX = ("fox", Fox, -1)
    BEE = ("bee", Bee, -1)
    HOGLIN = ("hoglin", Hoglin, -1)
    PIGLIN = ("piglin", Piglin, -1)
    STRIDER = ("strider", Strider, -1)
    ZOGLIN = ("zoglin", Zoglin, -1)
    PIGLIN_BRUTE = ("piglin_brute", PiglinBrute, -1)
    AXOLOTL = ("axolotl", Axolotl, -1)
    GLOW_ITEM_FRAME = ("glow_item_frame", GlowItemFrame, -1)
    GLOW_SQUID = ("glow_squid", GlowSquid, -1)
    GOAT = ("goat", Goat, -1)
    MARKER = ("marker", Marker, -1)
    ALLAY = ("allay", Allay, -1)
    FROG = ("frog", Frog, -1)
    TADPOLE = ("tadpole", Tadpole, -1)
    WARDEN = ("warden", Warden, -1)
    CAMEL = ("camel", Camel, -1)
    BLOCK_DISPLAY = ("block_display", BlockDisplay, -1)
    INTERACTION = ("interaction", Interaction, -1)
    ITEM_DISPLAY = ("item_display", ItemDisplay, -1)
    SNIFFER = ("sniffer", Sniffer, -1)
    TEXT_DISPLAY = ("text_display", TextDisplay, -1)
    BREEZE = ("breeze", Breeze, -1)
    WIND_CHARGE = ("wind_charge", WindCharge, -1)
    BREEZE_WIND_CHARGE = ("breeze_wind_charge", BreezeWindCharge, -1)
    ARMADILLO = ("armadillo", Armadillo, -1)
    BOGGED = ("bogged", Bogged, -1)
    OMINOUS_ITEM_SPAWNER = ("ominous_item_spawner", OminousItemSpawner, -1)
    ACACIA_BOAT = ("acacia_boat", AcaciaBoat, -1)
    ACACIA_CHEST_BOAT = ("acacia_chest_boat", AcaciaChestBoat, -1)
    BAMBOO_RAFT = ("bamboo_raft", BambooRaft, -1)
    BAMBOO_CHEST_RAFT = ("bamboo_chest_raft", BambooChestRaft, -1)
    BIRCH_BOAT = ("birch_boat", BirchBoat, -1)
    BIRCH_CHEST_BOAT = ("birch_chest_boat", BirchChestBoat, -1)
    CHERRY_BOAT = ("cherry_boat", CherryBoat, -1)
    CHERRY_CHEST_BOAT = ("cherry_chest_boat", CherryChestBoat, -1)
    DARK_OAK_BOAT = ("dark_oak_boat", DarkOakBoat, -1)
    DARK_OAK_CHEST_BOAT = ("dark_oak_chest_boat", DarkOakChestBoat, -1)
    JUNGLE_BOAT = ("jungle_boat", JungleBoat, -1)
    JUNGLE_CHEST_BOAT = ("jungle_chest_boat", JungleChestBoat, -1)
    MANGROVE_BOAT = ("mangrove_boat", MangroveBoat, -1)
    MANGROVE_CHEST_BOAT = ("mangrove_chest_boat", MangroveChestBoat, -1)
    OAK_BOAT = ("oak_boat", OakBoat, -1)
    OAK_CHEST_BOAT = ("oak_chest_boat", OakChestBoat, -1)
    PALE_OAK_BOAT = ("pale_oak_boat", PaleOakBoat, -1)
    PALE_OAK_CHEST_BOAT = ("pale_oak_chest_boat", PaleOakChestBoat, -1)
    SPRUCE_BOAT = ("spruce_boat", SpruceBoat, -1)
    SPRUCE_CHEST_BOAT = ("spruce_chest_boat", SpruceChestBoat, -1)
    CREAKING = ("creaking", Creaking, -1)
    FISHING_BOBBER = ("fishing_bobber", FishHook, -1, False)
    """
    A fishing line and bobber.
    """
    LIGHTNING_BOLT = ("lightning_bolt", LightningStrike, -1)
    """
    A bolt of lightning.
    
    Spawn with World.strikeLightning(Location).
    """
    PLAYER = ("player", Player, -1, False)
    UNKNOWN = (None, None, -1, False)
    """
    An unknown entity without an Entity Class
    """


    def getName(self) -> str:
        """
        Gets the entity type name.

        Returns
        - the entity type's name

        Deprecated
        - Magic value
        """
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...


    def getEntityClass(self) -> type["Entity"]:
        ...


    def getTypeId(self) -> int:
        """
        Gets the entity type id.

        Returns
        - the raw type id

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def fromName(name: str) -> "EntityType":
        """
        Gets an entity type from its name.

        Arguments
        - name: the entity type's name

        Returns
        - the matching entity type or null

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def fromId(id: int) -> "EntityType":
        """
        Gets an entity from its id.

        Arguments
        - id: the raw type id

        Returns
        - the matching entity type or null

        Deprecated
        - Magic value
        """
        ...


    def isSpawnable(self) -> bool:
        """
        Some entities cannot be spawned using World.spawnEntity(Location, EntityType) or World.spawn(Location, Class), usually because they require additional
        information in order to spawn.

        Returns
        - False if the entity type cannot be spawned
        """
        ...


    def isAlive(self) -> bool:
        ...


    def getTranslationKey(self) -> str:
        ...


    def isEnabledByFeature(self, world: "World") -> bool:
        """
        Gets if this EntityType is enabled by feature in a world.

        Arguments
        - world: the world to check

        Returns
        - True if this EntityType can be used to spawn an Entity for this World.
        """
        ...


    def getKeyOrThrow(self) -> "NamespacedKey":
        ...


    def getKeyOrNull(self) -> "NamespacedKey":
        ...


    def isRegistered(self) -> bool:
        ...



"""
Python module generated from Java source file org.bukkit.event.world.GenericGameEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit import GameEvent
from org.bukkit import Location
from org.bukkit.entity import Entity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.world import *
from typing import Any, Callable, Iterable, Tuple


class GenericGameEvent(WorldEvent, Cancellable):
    """
    Represents a generic Mojang game event.
    
    Specific Bukkit events should be used where possible, this event is mainly
    used internally by Sculk sensors.
    """

    def __init__(self, event: "GameEvent", location: "Location", entity: "Entity", radius: int, isAsync: bool):
        ...


    def getEvent(self) -> "GameEvent":
        """
        Get the underlying event.

        Returns
        - the event
        """
        ...


    def getLocation(self) -> "Location":
        """
        Get the location where the event occurred.

        Returns
        - event location
        """
        ...


    def getEntity(self) -> "Entity":
        """
        Get the entity which triggered this event, if present.

        Returns
        - triggering entity or null
        """
        ...


    def getRadius(self) -> int:
        """
        Get the block radius to which this event will be broadcast.

        Returns
        - broadcast radius
        """
        ...


    def setRadius(self, radius: int) -> None:
        """
        Set the radius to which the event should be broadcast.

        Arguments
        - radius: radius, must be greater than or equal to 0
        """
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def isCancelled(self) -> bool:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file java.util.concurrent.AbstractExecutorService

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Iterator
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class AbstractExecutorService(ExecutorService):
    """
    Provides default implementations of ExecutorService
    execution methods. This class implements the `submit`,
    `invokeAny` and `invokeAll` methods using a
    RunnableFuture returned by `newTaskFor`, which defaults
    to the FutureTask class provided in this package.  For example,
    the implementation of `submit(Runnable)` creates an
    associated `RunnableFuture` that is executed and
    returned. Subclasses may override the `newTaskFor` methods
    to return `RunnableFuture` implementations other than
    `FutureTask`.
    
    **Extension example.** Here is a sketch of a class
    that customizes ThreadPoolExecutor to use
    a `CustomTask` class instead of the default `FutureTask`:
    ``` `public class CustomThreadPoolExecutor extends ThreadPoolExecutor {
    
      static class CustomTask<V> implements RunnableFuture<V> { ...`
    
      protected <V> RunnableFuture<V> newTaskFor(Callable<V> c) {
          return new CustomTask<V>(c);
      }
      protected <V> RunnableFuture<V> newTaskFor(Runnable r, V v) {
          return new CustomTask<V>(r, v);
      }
      // ... add constructors, etc.
    }}```

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self):
        """
        Constructor for subclasses to call.
        """
        ...


    def submit(self, task: "Runnable") -> "Future"[Any]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def submit(self, task: "Runnable", result: "T") -> "Future"["T"]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def submit(self, task: "Callable"["T"]) -> "Future"["T"]:
        """
        Raises
        - RejectedExecutionException: 
        - NullPointerException: 
        """
        ...


    def invokeAny(self, tasks: Iterable["Callable"["T"]]) -> "T":
        ...


    def invokeAny(self, tasks: Iterable["Callable"["T"]], timeout: int, unit: "TimeUnit") -> "T":
        ...


    def invokeAll(self, tasks: Iterable["Callable"["T"]]) -> list["Future"["T"]]:
        ...


    def invokeAll(self, tasks: Iterable["Callable"["T"]], timeout: int, unit: "TimeUnit") -> list["Future"["T"]]:
        ...



"""
Python module generated from Java source file java.util.function.LongBinaryOperator

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.function import *
from typing import Any, Callable, Iterable, Tuple


class LongBinaryOperator:
    """
    Represents an operation upon two `long`-valued operands and producing a
    `long`-valued result.   This is the primitive type specialization of
    BinaryOperator for `long`.
    
    This is a <a href="package-summary.html">functional interface</a>
    whose functional method is .applyAsLong(long, long).

    See
    - LongUnaryOperator

    Since
    - 1.8
    """

    def applyAsLong(self, left: int, right: int) -> int:
        """
        Applies this operator to the given operands.

        Arguments
        - left: the first operand
        - right: the second operand

        Returns
        - the operator result
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.ConcurrentHashMap

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import ObjectStreamField
from java.io import Serializable
from java.lang.reflect import ParameterizedType
from java.lang.reflect import Type
from java.util import Arrays
from java.util import Enumeration
from java.util import Hashtable
from java.util import Iterator
from java.util import NoSuchElementException
from java.util import Spliterator
from java.util.concurrent import *
from java.util.concurrent.atomic import AtomicReference
from java.util.concurrent.locks import LockSupport
from java.util.concurrent.locks import ReentrantLock
from java.util.function import BiConsumer
from java.util.function import BiFunction
from java.util.function import Consumer
from java.util.function import DoubleBinaryOperator
from java.util.function import Function
from java.util.function import IntBinaryOperator
from java.util.function import LongBinaryOperator
from java.util.function import Predicate
from java.util.function import ToDoubleBiFunction
from java.util.function import ToDoubleFunction
from java.util.function import ToIntBiFunction
from java.util.function import ToIntFunction
from java.util.function import ToLongBiFunction
from java.util.function import ToLongFunction
from java.util.stream import Stream
from jdk.internal.misc import Unsafe
from typing import Any, Callable, Iterable, Tuple


class ConcurrentHashMap(AbstractMap, ConcurrentMap, Serializable):
    """
    A hash table supporting full concurrency of retrievals and
    high expected concurrency for updates. This class obeys the
    same functional specification as java.util.Hashtable, and
    includes versions of methods corresponding to each method of
    `Hashtable`. However, even though all operations are
    thread-safe, retrieval operations do *not* entail locking,
    and there is *not* any support for locking the entire table
    in a way that prevents all access.  This class is fully
    interoperable with `Hashtable` in programs that rely on its
    thread safety but not on its synchronization details.
    
    Retrieval operations (including `get`) generally do not
    block, so may overlap with update operations (including `put`
    and `remove`). Retrievals reflect the results of the most
    recently *completed* update operations holding upon their
    onset. (More formally, an update operation for a given key bears a
    *happens-before* relation with any (non-null) retrieval for
    that key reporting the updated value.)  For aggregate operations
    such as `putAll` and `clear`, concurrent retrievals may
    reflect insertion or removal of only some entries.  Similarly,
    Iterators, Spliterators and Enumerations return elements reflecting the
    state of the hash table at some point at or since the creation of the
    iterator/enumeration.  They do *not* throw java.util.ConcurrentModificationException ConcurrentModificationException.
    However, iterators are designed to be used by only one thread at a time.
    Bear in mind that the results of aggregate status methods including
    `size`, `isEmpty`, and `containsValue` are typically
    useful only when a map is not undergoing concurrent updates in other threads.
    Otherwise the results of these methods reflect transient states
    that may be adequate for monitoring or estimation purposes, but not
    for program control.
    
    The table is dynamically expanded when there are too many
    collisions (i.e., keys that have distinct hash codes but fall into
    the same slot modulo the table size), with the expected average
    effect of maintaining roughly two bins per mapping (corresponding
    to a 0.75 load factor threshold for resizing). There may be much
    variance around this average as mappings are added and removed, but
    overall, this maintains a commonly accepted time/space tradeoff for
    hash tables.  However, resizing this or any other kind of hash
    table may be a relatively slow operation. When possible, it is a
    good idea to provide a size estimate as an optional `initialCapacity` constructor argument. An additional optional
    `loadFactor` constructor argument provides a further means of
    customizing initial table capacity by specifying the table density
    to be used in calculating the amount of space to allocate for the
    given number of elements.  Also, for compatibility with previous
    versions of this class, constructors may optionally specify an
    expected `concurrencyLevel` as an additional hint for
    internal sizing.  Note that using many keys with exactly the same
    `hashCode()` is a sure way to slow down performance of any
    hash table. To ameliorate impact, when keys are Comparable,
    this class may use comparison order among keys to help break ties.
    
    A Set projection of a ConcurrentHashMap may be created
    (using .newKeySet() or .newKeySet(int)), or viewed
    (using .keySet(Object) when only keys are of interest, and the
    mapped values are (perhaps transiently) not used or all take the
    same mapping value.
    
    A ConcurrentHashMap can be used as a scalable frequency map (a
    form of histogram or multiset) by using java.util.concurrent.atomic.LongAdder values and initializing via
    .computeIfAbsent computeIfAbsent. For example, to add a count
    to a `ConcurrentHashMap<String,LongAdder> freqs`, you can use
    `freqs.computeIfAbsent(key, k -> new LongAdder()).increment();`
    
    This class and its views and iterators implement all of the
    *optional* methods of the Map and Iterator
    interfaces.
    
    Like Hashtable but unlike HashMap, this class
    does *not* allow `null` to be used as a key or value.
    
    ConcurrentHashMaps support a set of sequential and parallel bulk
    operations that, unlike most Stream methods, are designed
    to be safely, and often sensibly, applied even with maps that are
    being concurrently updated by other threads; for example, when
    computing a snapshot summary of the values in a shared registry.
    There are three kinds of operation, each with four forms, accepting
    functions with keys, values, entries, and (key, value) pairs as
    arguments and/or return values. Because the elements of a
    ConcurrentHashMap are not ordered in any particular way, and may be
    processed in different orders in different parallel executions, the
    correctness of supplied functions should not depend on any
    ordering, or on any other objects or values that may transiently
    change while computation is in progress; and except for forEach
    actions, should ideally be side-effect-free. Bulk operations on
    Map.Entry objects do not support method `setValue`.
    
    
    - forEach: Performs a given action on each element.
    A variant form applies a given transformation on each element
    before performing the action.
    
    - search: Returns the first available non-null result of
    applying a given function on each element; skipping further
    search when a result is found.
    
    - reduce: Accumulates each element.  The supplied reduction
    function cannot rely on ordering (more formally, it should be
    both associative and commutative).  There are five variants:
    
    
    
    - Plain reductions. (There is not a form of this method for
    (key, value) function arguments since there is no corresponding
    return type.)
    
    - Mapped reductions that accumulate the results of a given
    function applied to each element.
    
    - Reductions to scalar doubles, longs, and ints, using a
    given basis value.
    
    
    
    
    These bulk operations accept a `parallelismThreshold`
    argument. Methods proceed sequentially if the current map size is
    estimated to be less than the given threshold. Using a value of
    `Long.MAX_VALUE` suppresses all parallelism.  Using a value
    of `1` results in maximal parallelism by partitioning into
    enough subtasks to fully utilize the ForkJoinPool.commonPool() that is used for all parallel
    computations. Normally, you would initially choose one of these
    extreme values, and then measure performance of using in-between
    values that trade off overhead versus throughput.
    
    The concurrency properties of bulk operations follow
    from those of ConcurrentHashMap: Any non-null result returned
    from `get(key)` and related access methods bears a
    happens-before relation with the associated insertion or
    update.  The result of any bulk operation reflects the
    composition of these per-element relations (but is not
    necessarily atomic with respect to the map as a whole unless it
    is somehow known to be quiescent).  Conversely, because keys
    and values in the map are never null, null serves as a reliable
    atomic indicator of the current lack of any result.  To
    maintain this property, null serves as an implicit basis for
    all non-scalar reduction operations. For the double, long, and
    int versions, the basis should be one that, when combined with
    any other value, returns that other value (more formally, it
    should be the identity element for the reduction). Most common
    reductions have these properties; for example, computing a sum
    with basis 0 or a minimum with basis MAX_VALUE.
    
    Search and transformation functions provided as arguments
    should similarly return null to indicate the lack of any result
    (in which case it is not used). In the case of mapped
    reductions, this also enables transformations to serve as
    filters, returning null (or, in the case of primitive
    specializations, the identity basis) if the element should not
    be combined. You can create compound transformations and
    filterings by composing them yourself under this "null means
    there is nothing there now" rule before using them in search or
    reduce operations.
    
    Methods accepting and/or returning Entry arguments maintain
    key-value associations. They may be useful for example when
    finding the key for the greatest value. Note that "plain" Entry
    arguments can be supplied using `new
    AbstractMap.SimpleEntry(k,v)`.
    
    Bulk operations may complete abruptly, throwing an
    exception encountered in the application of a supplied
    function. Bear in mind when handling such exceptions that other
    concurrently executing functions could also have thrown
    exceptions, or would have done so if the first exception had
    not occurred.
    
    Speedups for parallel compared to sequential forms are common
    but not guaranteed.  Parallel operations involving brief functions
    on small maps may execute more slowly than sequential forms if the
    underlying work to parallelize the computation is more expensive
    than the computation itself.  Similarly, parallelization may not
    lead to much actual parallelism if all processors are busy
    performing unrelated tasks.
    
    All arguments to all task methods must be non-null.
    
    This class is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<K>`: the type of keys maintained by this map
    
    Type `<V>`: the type of mapped values

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self):
        """
        Creates a new, empty map with the default initial table size (16).
        """
        ...


    def __init__(self, initialCapacity: int):
        """
        Creates a new, empty map with an initial table size
        accommodating the specified number of elements without the need
        to dynamically resize.

        Arguments
        - initialCapacity: The implementation performs internal
        sizing to accommodate this many elements.

        Raises
        - IllegalArgumentException: if the initial capacity of
        elements is negative
        """
        ...


    def __init__(self, m: dict["K", "V"]):
        """
        Creates a new map with the same mappings as the given map.

        Arguments
        - m: the map
        """
        ...


    def __init__(self, initialCapacity: int, loadFactor: float):
        """
        Creates a new, empty map with an initial table size based on
        the given number of elements (`initialCapacity`) and
        initial table density (`loadFactor`).

        Arguments
        - initialCapacity: the initial capacity. The implementation
        performs internal sizing to accommodate this many elements,
        given the specified load factor.
        - loadFactor: the load factor (table density) for
        establishing the initial table size

        Raises
        - IllegalArgumentException: if the initial capacity of
        elements is negative or the load factor is nonpositive

        Since
        - 1.6
        """
        ...


    def __init__(self, initialCapacity: int, loadFactor: float, concurrencyLevel: int):
        """
        Creates a new, empty map with an initial table size based on
        the given number of elements (`initialCapacity`), initial
        table density (`loadFactor`), and number of concurrently
        updating threads (`concurrencyLevel`).

        Arguments
        - initialCapacity: the initial capacity. The implementation
        performs internal sizing to accommodate this many elements,
        given the specified load factor.
        - loadFactor: the load factor (table density) for
        establishing the initial table size
        - concurrencyLevel: the estimated number of concurrently
        updating threads. The implementation may use this value as
        a sizing hint.

        Raises
        - IllegalArgumentException: if the initial capacity is
        negative or the load factor or concurrencyLevel are
        nonpositive
        """
        ...


    def size(self) -> int:
        """

        """
        ...


    def isEmpty(self) -> bool:
        """

        """
        ...


    def get(self, key: "Object") -> "V":
        """
        Returns the value to which the specified key is mapped,
        or `null` if this map contains no mapping for the key.
        
        More formally, if this map contains a mapping from a key
        `k` to a value `v` such that `key.equals(k)`,
        then this method returns `v`; otherwise it returns
        `null`.  (There can be at most one such mapping.)

        Raises
        - NullPointerException: if the specified key is null
        """
        ...


    def containsKey(self, key: "Object") -> bool:
        """
        Tests if the specified object is a key in this table.

        Arguments
        - key: possible key

        Returns
        - `True` if and only if the specified object
                is a key in this table, as determined by the
                `equals` method; `False` otherwise

        Raises
        - NullPointerException: if the specified key is null
        """
        ...


    def containsValue(self, value: "Object") -> bool:
        """
        Returns `True` if this map maps one or more keys to the
        specified value. Note: This method may require a full traversal
        of the map, and is much slower than method `containsKey`.

        Arguments
        - value: value whose presence in this map is to be tested

        Returns
        - `True` if this map maps one or more keys to the
                specified value

        Raises
        - NullPointerException: if the specified value is null
        """
        ...


    def put(self, key: "K", value: "V") -> "V":
        """
        Maps the specified key to the specified value in this table.
        Neither the key nor the value can be null.
        
        The value can be retrieved by calling the `get` method
        with a key that is equal to the original key.

        Arguments
        - key: key with which the specified value is to be associated
        - value: value to be associated with the specified key

        Returns
        - the previous value associated with `key`, or
                `null` if there was no mapping for `key`

        Raises
        - NullPointerException: if the specified key or value is null
        """
        ...


    def putAll(self, m: dict["K", "V"]) -> None:
        """
        Copies all of the mappings from the specified map to this one.
        These mappings replace any mappings that this map had for any of the
        keys currently in the specified map.

        Arguments
        - m: mappings to be stored in this map
        """
        ...


    def remove(self, key: "Object") -> "V":
        """
        Removes the key (and its corresponding value) from this map.
        This method does nothing if the key is not in the map.

        Arguments
        - key: the key that needs to be removed

        Returns
        - the previous value associated with `key`, or
                `null` if there was no mapping for `key`

        Raises
        - NullPointerException: if the specified key is null
        """
        ...


    def clear(self) -> None:
        """
        Removes all of the mappings from this map.
        """
        ...


    def keySet(self) -> "KeySetView"["K", "V"]:
        """
        Returns a Set view of the keys contained in this map.
        The set is backed by the map, so changes to the map are
        reflected in the set, and vice-versa. The set supports element
        removal, which removes the corresponding mapping from this map,
        via the `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear`
        operations.  It does not support the `add` or
        `addAll` operations.
        
        The view's iterators and spliterators are
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.
        
        The view's `spliterator` reports Spliterator.CONCURRENT,
        Spliterator.DISTINCT, and Spliterator.NONNULL.

        Returns
        - the set view
        """
        ...


    def values(self) -> Iterable["V"]:
        """
        Returns a Collection view of the values contained in this map.
        The collection is backed by the map, so changes to the map are
        reflected in the collection, and vice-versa.  The collection
        supports element removal, which removes the corresponding
        mapping from this map, via the `Iterator.remove`,
        `Collection.remove`, `removeAll`,
        `retainAll`, and `clear` operations.  It does not
        support the `add` or `addAll` operations.
        
        The view's iterators and spliterators are
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.
        
        The view's `spliterator` reports Spliterator.CONCURRENT
        and Spliterator.NONNULL.

        Returns
        - the collection view
        """
        ...


    def entrySet(self) -> set["Map.Entry"["K", "V"]]:
        """
        Returns a Set view of the mappings contained in this map.
        The set is backed by the map, so changes to the map are
        reflected in the set, and vice-versa.  The set supports element
        removal, which removes the corresponding mapping from the map,
        via the `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear`
        operations.
        
        The view's iterators and spliterators are
        <a href="package-summary.html#Weakly">*weakly consistent*</a>.
        
        The view's `spliterator` reports Spliterator.CONCURRENT,
        Spliterator.DISTINCT, and Spliterator.NONNULL.

        Returns
        - the set view
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code value for this Map, i.e.,
        the sum of, for each key-value pair in the map,
        `key.hashCode() ^ value.hashCode()`.

        Returns
        - the hash code value for this map
        """
        ...


    def toString(self) -> str:
        """
        Returns a string representation of this map.  The string
        representation consists of a list of key-value mappings (in no
        particular order) enclosed in braces ("`{`}").  Adjacent
        mappings are separated by the characters `", "` (comma
        and space).  Each key-value mapping is rendered as the key
        followed by an equals sign ("`=`") followed by the
        associated value.

        Returns
        - a string representation of this map
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Compares the specified object with this map for equality.
        Returns `True` if the given object is a map with the same
        mappings as this map.  This operation may return misleading
        results if either map is concurrently modified during execution
        of this method.

        Arguments
        - o: object to be compared for equality with this map

        Returns
        - `True` if the specified object is equal to this map
        """
        ...


    def putIfAbsent(self, key: "K", value: "V") -> "V":
        """
        Returns
        - the previous value associated with the specified key,
                or `null` if there was no mapping for the key

        Raises
        - NullPointerException: if the specified key or value is null
        """
        ...


    def remove(self, key: "Object", value: "Object") -> bool:
        """
        Raises
        - NullPointerException: if the specified key is null
        """
        ...


    def replace(self, key: "K", oldValue: "V", newValue: "V") -> bool:
        """
        Raises
        - NullPointerException: if any of the arguments are null
        """
        ...


    def replace(self, key: "K", value: "V") -> "V":
        """
        Returns
        - the previous value associated with the specified key,
                or `null` if there was no mapping for the key

        Raises
        - NullPointerException: if the specified key or value is null
        """
        ...


    def getOrDefault(self, key: "Object", defaultValue: "V") -> "V":
        """
        Returns the value to which the specified key is mapped, or the
        given default value if this map contains no mapping for the
        key.

        Arguments
        - key: the key whose associated value is to be returned
        - defaultValue: the value to return if this map contains
        no mapping for the given key

        Returns
        - the mapping for the key, if present; else the default value

        Raises
        - NullPointerException: if the specified key is null
        """
        ...


    def forEach(self, action: "BiConsumer"["K", "V"]) -> None:
        ...


    def replaceAll(self, function: "BiFunction"["K", "V", "V"]) -> None:
        ...


    def computeIfAbsent(self, key: "K", mappingFunction: "Function"["K", "V"]) -> "V":
        """
        If the specified key is not already associated with a value,
        attempts to compute its value using the given mapping function
        and enters it into this map unless `null`.  The entire
        method invocation is performed atomically.  The supplied
        function is invoked exactly once per invocation of this method
        if the key is absent, else not at all.  Some attempted update
        operations on this map by other threads may be blocked while
        computation is in progress, so the computation should be short
        and simple.
        
        The mapping function must not modify this map during computation.

        Arguments
        - key: key with which the specified value is to be associated
        - mappingFunction: the function to compute a value

        Returns
        - the current (existing or computed) value associated with
                the specified key, or null if the computed value is null

        Raises
        - NullPointerException: if the specified key or mappingFunction
                is null
        - IllegalStateException: if the computation detectably
                attempts a recursive update to this map that would
                otherwise never complete
        - RuntimeException: or Error if the mappingFunction does so,
                in which case the mapping is left unestablished
        """
        ...


    def computeIfPresent(self, key: "K", remappingFunction: "BiFunction"["K", "V", "V"]) -> "V":
        """
        If the value for the specified key is present, attempts to
        compute a new mapping given the key and its current mapped
        value.  The entire method invocation is performed atomically.
        The supplied function is invoked exactly once per invocation of
        this method if the key is present, else not at all.  Some
        attempted update operations on this map by other threads may be
        blocked while computation is in progress, so the computation
        should be short and simple.
        
        The remapping function must not modify this map during computation.

        Arguments
        - key: key with which a value may be associated
        - remappingFunction: the function to compute a value

        Returns
        - the new value associated with the specified key, or null if none

        Raises
        - NullPointerException: if the specified key or remappingFunction
                is null
        - IllegalStateException: if the computation detectably
                attempts a recursive update to this map that would
                otherwise never complete
        - RuntimeException: or Error if the remappingFunction does so,
                in which case the mapping is unchanged
        """
        ...


    def compute(self, key: "K", remappingFunction: "BiFunction"["K", "V", "V"]) -> "V":
        """
        Attempts to compute a mapping for the specified key and its
        current mapped value (or `null` if there is no current
        mapping). The entire method invocation is performed atomically.
        The supplied function is invoked exactly once per invocation of
        this method.  Some attempted update operations on this map by
        other threads may be blocked while computation is in progress,
        so the computation should be short and simple.
        
        The remapping function must not modify this map during computation.

        Arguments
        - key: key with which the specified value is to be associated
        - remappingFunction: the function to compute a value

        Returns
        - the new value associated with the specified key, or null if none

        Raises
        - NullPointerException: if the specified key or remappingFunction
                is null
        - IllegalStateException: if the computation detectably
                attempts a recursive update to this map that would
                otherwise never complete
        - RuntimeException: or Error if the remappingFunction does so,
                in which case the mapping is unchanged
        """
        ...


    def merge(self, key: "K", value: "V", remappingFunction: "BiFunction"["V", "V", "V"]) -> "V":
        """
        If the specified key is not already associated with a
        (non-null) value, associates it with the given value.
        Otherwise, replaces the value with the results of the given
        remapping function, or removes if `null`. The entire
        method invocation is performed atomically.  Some attempted
        update operations on this map by other threads may be blocked
        while computation is in progress, so the computation should be
        short and simple, and must not attempt to update any other
        mappings of this Map.

        Arguments
        - key: key with which the specified value is to be associated
        - value: the value to use if absent
        - remappingFunction: the function to recompute a value if present

        Returns
        - the new value associated with the specified key, or null if none

        Raises
        - NullPointerException: if the specified key or the
                remappingFunction is null
        - RuntimeException: or Error if the remappingFunction does so,
                in which case the mapping is unchanged
        """
        ...


    def contains(self, value: "Object") -> bool:
        """
        Tests if some key maps into the specified value in this table.
        
        Note that this method is identical in functionality to
        .containsValue(Object), and exists solely to ensure
        full compatibility with class java.util.Hashtable,
        which supported this method prior to introduction of the
        Java Collections Framework.

        Arguments
        - value: a value to search for

        Returns
        - `True` if and only if some key maps to the
                `value` argument in this table as
                determined by the `equals` method;
                `False` otherwise

        Raises
        - NullPointerException: if the specified value is null
        """
        ...


    def keys(self) -> "Enumeration"["K"]:
        """
        Returns an enumeration of the keys in this table.

        Returns
        - an enumeration of the keys in this table

        See
        - .keySet()
        """
        ...


    def elements(self) -> "Enumeration"["V"]:
        """
        Returns an enumeration of the values in this table.

        Returns
        - an enumeration of the values in this table

        See
        - .values()
        """
        ...


    def mappingCount(self) -> int:
        """
        Returns the number of mappings. This method should be used
        instead of .size because a ConcurrentHashMap may
        contain more mappings than can be represented as an int. The
        value returned is an estimate; the actual count may differ if
        there are concurrent insertions or removals.

        Returns
        - the number of mappings

        Since
        - 1.8
        """
        ...


    @staticmethod
    def newKeySet() -> "KeySetView"["K", "Boolean"]:
        """
        Creates a new Set backed by a ConcurrentHashMap
        from the given type to `Boolean.TRUE`.
        
        Type `<K>`: the element type of the returned set

        Returns
        - the new set

        Since
        - 1.8
        """
        ...


    @staticmethod
    def newKeySet(initialCapacity: int) -> "KeySetView"["K", "Boolean"]:
        """
        Creates a new Set backed by a ConcurrentHashMap
        from the given type to `Boolean.TRUE`.
        
        Type `<K>`: the element type of the returned set

        Arguments
        - initialCapacity: The implementation performs internal
        sizing to accommodate this many elements.

        Returns
        - the new set

        Raises
        - IllegalArgumentException: if the initial capacity of
        elements is negative

        Since
        - 1.8
        """
        ...


    def keySet(self, mappedValue: "V") -> "KeySetView"["K", "V"]:
        """
        Returns a Set view of the keys in this map, using the
        given common mapped value for any additions (i.e., Collection.add and Collection.addAll(Collection)).
        This is of course only appropriate if it is acceptable to use
        the same value for all additions from this view.

        Arguments
        - mappedValue: the mapped value to use for any additions

        Returns
        - the set view

        Raises
        - NullPointerException: if the mappedValue is null
        """
        ...


    def forEach(self, parallelismThreshold: int, action: "BiConsumer"["K", "V"]) -> None:
        """
        Performs the given action for each (key, value).

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - action: the action

        Since
        - 1.8
        """
        ...


    def forEach(self, parallelismThreshold: int, transformer: "BiFunction"["K", "V", "U"], action: "Consumer"["U"]) -> None:
        """
        Performs the given action for each non-null transformation
        of each (key, value).
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case the action is not applied)
        - action: the action

        Since
        - 1.8
        """
        ...


    def search(self, parallelismThreshold: int, searchFunction: "BiFunction"["K", "V", "U"]) -> "U":
        """
        Returns a non-null result from applying the given search
        function on each (key, value), or null if none.  Upon
        success, further element processing is suppressed and the
        results of any other parallel invocations of the search
        function are ignored.
        
        Type `<U>`: the return type of the search function

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - searchFunction: a function returning a non-null
        result on success, else null

        Returns
        - a non-null result from applying the given search
        function on each (key, value), or null if none

        Since
        - 1.8
        """
        ...


    def reduce(self, parallelismThreshold: int, transformer: "BiFunction"["K", "V", "U"], reducer: "BiFunction"["U", "U", "U"]) -> "U":
        """
        Returns the result of accumulating the given transformation
        of all (key, value) pairs using the given reducer to
        combine values, or null if none.
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case it is not combined)
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all (key, value) pairs

        Since
        - 1.8
        """
        ...


    def reduceToDouble(self, parallelismThreshold: int, transformer: "ToDoubleBiFunction"["K", "V"], basis: float, reducer: "DoubleBinaryOperator") -> float:
        """
        Returns the result of accumulating the given transformation
        of all (key, value) pairs using the given reducer to
        combine values, and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all (key, value) pairs

        Since
        - 1.8
        """
        ...


    def reduceToLong(self, parallelismThreshold: int, transformer: "ToLongBiFunction"["K", "V"], basis: int, reducer: "LongBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all (key, value) pairs using the given reducer to
        combine values, and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all (key, value) pairs

        Since
        - 1.8
        """
        ...


    def reduceToInt(self, parallelismThreshold: int, transformer: "ToIntBiFunction"["K", "V"], basis: int, reducer: "IntBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all (key, value) pairs using the given reducer to
        combine values, and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all (key, value) pairs

        Since
        - 1.8
        """
        ...


    def forEachKey(self, parallelismThreshold: int, action: "Consumer"["K"]) -> None:
        """
        Performs the given action for each key.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - action: the action

        Since
        - 1.8
        """
        ...


    def forEachKey(self, parallelismThreshold: int, transformer: "Function"["K", "U"], action: "Consumer"["U"]) -> None:
        """
        Performs the given action for each non-null transformation
        of each key.
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case the action is not applied)
        - action: the action

        Since
        - 1.8
        """
        ...


    def searchKeys(self, parallelismThreshold: int, searchFunction: "Function"["K", "U"]) -> "U":
        """
        Returns a non-null result from applying the given search
        function on each key, or null if none. Upon success,
        further element processing is suppressed and the results of
        any other parallel invocations of the search function are
        ignored.
        
        Type `<U>`: the return type of the search function

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - searchFunction: a function returning a non-null
        result on success, else null

        Returns
        - a non-null result from applying the given search
        function on each key, or null if none

        Since
        - 1.8
        """
        ...


    def reduceKeys(self, parallelismThreshold: int, reducer: "BiFunction"["K", "K", "K"]) -> "K":
        """
        Returns the result of accumulating all keys using the given
        reducer to combine values, or null if none.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating all keys using the given
        reducer to combine values, or null if none

        Since
        - 1.8
        """
        ...


    def reduceKeys(self, parallelismThreshold: int, transformer: "Function"["K", "U"], reducer: "BiFunction"["U", "U", "U"]) -> "U":
        """
        Returns the result of accumulating the given transformation
        of all keys using the given reducer to combine values, or
        null if none.
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case it is not combined)
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all keys

        Since
        - 1.8
        """
        ...


    def reduceKeysToDouble(self, parallelismThreshold: int, transformer: "ToDoubleFunction"["K"], basis: float, reducer: "DoubleBinaryOperator") -> float:
        """
        Returns the result of accumulating the given transformation
        of all keys using the given reducer to combine values, and
        the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all keys

        Since
        - 1.8
        """
        ...


    def reduceKeysToLong(self, parallelismThreshold: int, transformer: "ToLongFunction"["K"], basis: int, reducer: "LongBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all keys using the given reducer to combine values, and
        the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all keys

        Since
        - 1.8
        """
        ...


    def reduceKeysToInt(self, parallelismThreshold: int, transformer: "ToIntFunction"["K"], basis: int, reducer: "IntBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all keys using the given reducer to combine values, and
        the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all keys

        Since
        - 1.8
        """
        ...


    def forEachValue(self, parallelismThreshold: int, action: "Consumer"["V"]) -> None:
        """
        Performs the given action for each value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - action: the action

        Since
        - 1.8
        """
        ...


    def forEachValue(self, parallelismThreshold: int, transformer: "Function"["V", "U"], action: "Consumer"["U"]) -> None:
        """
        Performs the given action for each non-null transformation
        of each value.
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case the action is not applied)
        - action: the action

        Since
        - 1.8
        """
        ...


    def searchValues(self, parallelismThreshold: int, searchFunction: "Function"["V", "U"]) -> "U":
        """
        Returns a non-null result from applying the given search
        function on each value, or null if none.  Upon success,
        further element processing is suppressed and the results of
        any other parallel invocations of the search function are
        ignored.
        
        Type `<U>`: the return type of the search function

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - searchFunction: a function returning a non-null
        result on success, else null

        Returns
        - a non-null result from applying the given search
        function on each value, or null if none

        Since
        - 1.8
        """
        ...


    def reduceValues(self, parallelismThreshold: int, reducer: "BiFunction"["V", "V", "V"]) -> "V":
        """
        Returns the result of accumulating all values using the
        given reducer to combine values, or null if none.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating all values

        Since
        - 1.8
        """
        ...


    def reduceValues(self, parallelismThreshold: int, transformer: "Function"["V", "U"], reducer: "BiFunction"["U", "U", "U"]) -> "U":
        """
        Returns the result of accumulating the given transformation
        of all values using the given reducer to combine values, or
        null if none.
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case it is not combined)
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all values

        Since
        - 1.8
        """
        ...


    def reduceValuesToDouble(self, parallelismThreshold: int, transformer: "ToDoubleFunction"["V"], basis: float, reducer: "DoubleBinaryOperator") -> float:
        """
        Returns the result of accumulating the given transformation
        of all values using the given reducer to combine values,
        and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all values

        Since
        - 1.8
        """
        ...


    def reduceValuesToLong(self, parallelismThreshold: int, transformer: "ToLongFunction"["V"], basis: int, reducer: "LongBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all values using the given reducer to combine values,
        and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all values

        Since
        - 1.8
        """
        ...


    def reduceValuesToInt(self, parallelismThreshold: int, transformer: "ToIntFunction"["V"], basis: int, reducer: "IntBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all values using the given reducer to combine values,
        and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all values

        Since
        - 1.8
        """
        ...


    def forEachEntry(self, parallelismThreshold: int, action: "Consumer"["Map.Entry"["K", "V"]]) -> None:
        """
        Performs the given action for each entry.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - action: the action

        Since
        - 1.8
        """
        ...


    def forEachEntry(self, parallelismThreshold: int, transformer: "Function"["Map.Entry"["K", "V"], "U"], action: "Consumer"["U"]) -> None:
        """
        Performs the given action for each non-null transformation
        of each entry.
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case the action is not applied)
        - action: the action

        Since
        - 1.8
        """
        ...


    def searchEntries(self, parallelismThreshold: int, searchFunction: "Function"["Map.Entry"["K", "V"], "U"]) -> "U":
        """
        Returns a non-null result from applying the given search
        function on each entry, or null if none.  Upon success,
        further element processing is suppressed and the results of
        any other parallel invocations of the search function are
        ignored.
        
        Type `<U>`: the return type of the search function

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - searchFunction: a function returning a non-null
        result on success, else null

        Returns
        - a non-null result from applying the given search
        function on each entry, or null if none

        Since
        - 1.8
        """
        ...


    def reduceEntries(self, parallelismThreshold: int, reducer: "BiFunction"["Map.Entry"["K", "V"], "Map.Entry"["K", "V"], "Map.Entry"["K", "V"]]) -> "Map.Entry"["K", "V"]:
        """
        Returns the result of accumulating all entries using the
        given reducer to combine values, or null if none.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating all entries

        Since
        - 1.8
        """
        ...


    def reduceEntries(self, parallelismThreshold: int, transformer: "Function"["Map.Entry"["K", "V"], "U"], reducer: "BiFunction"["U", "U", "U"]) -> "U":
        """
        Returns the result of accumulating the given transformation
        of all entries using the given reducer to combine values,
        or null if none.
        
        Type `<U>`: the return type of the transformer

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element, or null if there is no transformation (in
        which case it is not combined)
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all entries

        Since
        - 1.8
        """
        ...


    def reduceEntriesToDouble(self, parallelismThreshold: int, transformer: "ToDoubleFunction"["Map.Entry"["K", "V"]], basis: float, reducer: "DoubleBinaryOperator") -> float:
        """
        Returns the result of accumulating the given transformation
        of all entries using the given reducer to combine values,
        and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all entries

        Since
        - 1.8
        """
        ...


    def reduceEntriesToLong(self, parallelismThreshold: int, transformer: "ToLongFunction"["Map.Entry"["K", "V"]], basis: int, reducer: "LongBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all entries using the given reducer to combine values,
        and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all entries

        Since
        - 1.8
        """
        ...


    def reduceEntriesToInt(self, parallelismThreshold: int, transformer: "ToIntFunction"["Map.Entry"["K", "V"]], basis: int, reducer: "IntBinaryOperator") -> int:
        """
        Returns the result of accumulating the given transformation
        of all entries using the given reducer to combine values,
        and the given basis as an identity value.

        Arguments
        - parallelismThreshold: the (estimated) number of elements
        needed for this operation to be executed in parallel
        - transformer: a function returning the transformation
        for an element
        - basis: the identity (initial default value) for the reduction
        - reducer: a commutative associative combining function

        Returns
        - the result of accumulating the given transformation
        of all entries

        Since
        - 1.8
        """
        ...


    class KeySetView(CollectionView, Set, Serializable):
        """
        A view of a ConcurrentHashMap as a Set of keys, in
        which additions may optionally be enabled by mapping to a
        common value.  This class cannot be directly instantiated.
        See .keySet() keySet(),
        .keySet(Object) keySet(V),
        .newKeySet() newKeySet(),
        .newKeySet(int) newKeySet(int).

        Since
        - 1.8
        """

        def getMappedValue(self) -> "V":
            """
            Returns the default mapped value for additions,
            or `null` if additions are not supported.

            Returns
            - the default mapped value for additions, or `null`
            if not supported
            """
            ...


        def contains(self, o: "Object") -> bool:
            """
            Raises
            - NullPointerException: if the specified key is null
            """
            ...


        def remove(self, o: "Object") -> bool:
            """
            Removes the key from this map view, by removing the key (and its
            corresponding value) from the backing map.  This method does
            nothing if the key is not in the map.

            Arguments
            - o: the key to be removed from the backing map

            Returns
            - `True` if the backing map contained the specified key

            Raises
            - NullPointerException: if the specified key is null
            """
            ...


        def iterator(self) -> Iterator["K"]:
            """
            Returns
            - an iterator over the keys of the backing map
            """
            ...


        def add(self, e: "K") -> bool:
            """
            Adds the specified key to this set view by mapping the key to
            the default mapped value in the backing map, if defined.

            Arguments
            - e: key to be added

            Returns
            - `True` if this set changed as a result of the call

            Raises
            - NullPointerException: if the specified key is null
            - UnsupportedOperationException: if no default mapped value
            for additions was provided
            """
            ...


        def addAll(self, c: Iterable["K"]) -> bool:
            """
            Adds all of the elements in the specified collection to this set,
            as if by calling .add on each one.

            Arguments
            - c: the elements to be inserted into this set

            Returns
            - `True` if this set changed as a result of the call

            Raises
            - NullPointerException: if the collection or any of its
            elements are `null`
            - UnsupportedOperationException: if no default mapped value
            for additions was provided
            """
            ...


        def hashCode(self) -> int:
            ...


        def equals(self, o: "Object") -> bool:
            ...


        def spliterator(self) -> "Spliterator"["K"]:
            ...


        def forEach(self, action: "Consumer"["K"]) -> None:
            ...



"""
Python module generated from Java source file org.bukkit.inventory.view.StonecutterView

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import StonecutterInventory
from org.bukkit.inventory import StonecuttingRecipe
from org.bukkit.inventory.view import *
from typing import Any, Callable, Iterable, Tuple


class StonecutterView(InventoryView):
    """
    An instance of InventoryView which provides extra methods related to
    stonecutter view data.
    """

    def getTopInventory(self) -> "StonecutterInventory":
        ...


    def getSelectedRecipeIndex(self) -> int:
        """
        Gets the current index of the selected recipe.

        Returns
        - The index of the selected recipe in the stonecutter or -1 if null
        """
        ...


    def getRecipes(self) -> list["StonecuttingRecipe"]:
        """
        Gets a copy of all recipes currently available to the player.

        Returns
        - A copy of the StonecuttingRecipe's currently available
        for the player
        """
        ...


    def getRecipeAmount(self) -> int:
        """
        Gets the amount of recipes currently available.

        Returns
        - The amount of recipes currently available for the player
        """
        ...



"""
Python module generated from Java source file java.util.EnumMap

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from jdk.internal.access import SharedSecrets
from typing import Any, Callable, Iterable, Tuple


class EnumMap(AbstractMap, Serializable, Cloneable):
    """
    A specialized Map implementation for use with enum type keys.  All
    of the keys in an enum map must come from a single enum type that is
    specified, explicitly or implicitly, when the map is created.  Enum maps
    are represented internally as arrays.  This representation is extremely
    compact and efficient.
    
    Enum maps are maintained in the *natural order* of their keys
    (the order in which the enum constants are declared).  This is reflected
    in the iterators returned by the collections views (.keySet(),
    .entrySet(), and .values()).
    
    Iterators returned by the collection views are *weakly consistent*:
    they will never throw ConcurrentModificationException and they may
    or may not show the effects of any modifications to the map that occur while
    the iteration is in progress.
    
    Null keys are not permitted.  Attempts to insert a null key will
    throw NullPointerException.  Attempts to test for the
    presence of a null key or to remove one will, however, function properly.
    Null values are permitted.
    
    <P>Like most collection implementations `EnumMap` is not
    synchronized. If multiple threads access an enum map concurrently, and at
    least one of the threads modifies the map, it should be synchronized
    externally.  This is typically accomplished by synchronizing on some
    object that naturally encapsulates the enum map.  If no such object exists,
    the map should be "wrapped" using the Collections.synchronizedMap
    method.  This is best done at creation time, to prevent accidental
    unsynchronized access:
    
    ```
        Map&lt;EnumKey, V&gt; m
            = Collections.synchronizedMap(new EnumMap&lt;EnumKey, V&gt;(...));
    ```
    
    Implementation note: All basic operations execute in constant time.
    They are likely (though not guaranteed) to be faster than their
    HashMap counterparts.
    
    This class is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.

    Author(s)
    - Josh Bloch

    See
    - EnumSet

    Since
    - 1.5
    """

    def __init__(self, keyType: type["K"]):
        """
        Creates an empty enum map with the specified key type.

        Arguments
        - keyType: the class object of the key type for this enum map

        Raises
        - NullPointerException: if `keyType` is null
        """
        ...


    def __init__(self, m: "EnumMap"["K", "V"]):
        """
        Creates an enum map with the same key type as the specified enum
        map, initially containing the same mappings (if any).

        Arguments
        - m: the enum map from which to initialize this enum map

        Raises
        - NullPointerException: if `m` is null
        """
        ...


    def __init__(self, m: dict["K", "V"]):
        """
        Creates an enum map initialized from the specified map.  If the
        specified map is an `EnumMap` instance, this constructor behaves
        identically to .EnumMap(EnumMap).  Otherwise, the specified map
        must contain at least one mapping (in order to determine the new
        enum map's key type).

        Arguments
        - m: the map from which to initialize this enum map

        Raises
        - IllegalArgumentException: if `m` is not an
            `EnumMap` instance and contains no mappings
        - NullPointerException: if `m` is null
        """
        ...


    def size(self) -> int:
        """
        Returns the number of key-value mappings in this map.

        Returns
        - the number of key-value mappings in this map
        """
        ...


    def containsValue(self, value: "Object") -> bool:
        """
        Returns `True` if this map maps one or more keys to the
        specified value.

        Arguments
        - value: the value whose presence in this map is to be tested

        Returns
        - `True` if this map maps one or more keys to this value
        """
        ...


    def containsKey(self, key: "Object") -> bool:
        """
        Returns `True` if this map contains a mapping for the specified
        key.

        Arguments
        - key: the key whose presence in this map is to be tested

        Returns
        - `True` if this map contains a mapping for the specified
                   key
        """
        ...


    def get(self, key: "Object") -> "V":
        """
        Returns the value to which the specified key is mapped,
        or `null` if this map contains no mapping for the key.
        
        More formally, if this map contains a mapping from a key
        `k` to a value `v` such that `(key == k)`,
        then this method returns `v`; otherwise it returns
        `null`.  (There can be at most one such mapping.)
        
        A return value of `null` does not *necessarily*
        indicate that the map contains no mapping for the key; it's also
        possible that the map explicitly maps the key to `null`.
        The .containsKey containsKey operation may be used to
        distinguish these two cases.
        """
        ...


    def put(self, key: "K", value: "V") -> "V":
        """
        Associates the specified value with the specified key in this map.
        If the map previously contained a mapping for this key, the old
        value is replaced.

        Arguments
        - key: the key with which the specified value is to be associated
        - value: the value to be associated with the specified key

        Returns
        - the previous value associated with specified key, or
            `null` if there was no mapping for key.  (A `null`
            return can also indicate that the map previously associated
            `null` with the specified key.)

        Raises
        - NullPointerException: if the specified key is null
        """
        ...


    def remove(self, key: "Object") -> "V":
        """
        Removes the mapping for this key from this map if present.

        Arguments
        - key: the key whose mapping is to be removed from the map

        Returns
        - the previous value associated with specified key, or
            `null` if there was no entry for key.  (A `null`
            return can also indicate that the map previously associated
            `null` with the specified key.)
        """
        ...


    def putAll(self, m: dict["K", "V"]) -> None:
        """
        Copies all of the mappings from the specified map to this map.
        These mappings will replace any mappings that this map had for
        any of the keys currently in the specified map.

        Arguments
        - m: the mappings to be stored in this map

        Raises
        - NullPointerException: the specified map is null, or if
            one or more keys in the specified map are null
        """
        ...


    def clear(self) -> None:
        """
        Removes all mappings from this map.
        """
        ...


    def keySet(self) -> set["K"]:
        """
        Returns a Set view of the keys contained in this map.
        The returned set obeys the general contract outlined in
        Map.keySet().  The set's iterator will return the keys
        in their natural order (the order in which the enum constants
        are declared).

        Returns
        - a set view of the keys contained in this enum map
        """
        ...


    def values(self) -> Iterable["V"]:
        """
        Returns a Collection view of the values contained in this map.
        The returned collection obeys the general contract outlined in
        Map.values().  The collection's iterator will return the
        values in the order their corresponding keys appear in map,
        which is their natural order (the order in which the enum constants
        are declared).

        Returns
        - a collection view of the values contained in this map
        """
        ...


    def entrySet(self) -> set["Map.Entry"["K", "V"]]:
        """
        Returns a Set view of the mappings contained in this map.
        The returned set obeys the general contract outlined in
        Map.keySet().  The set's iterator will return the
        mappings in the order their keys appear in map, which is their
        natural order (the order in which the enum constants are declared).

        Returns
        - a set view of the mappings contained in this enum map
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Compares the specified object with this map for equality.  Returns
        `True` if the given object is also a map and the two maps
        represent the same mappings, as specified in the Map.equals(Object) contract.

        Arguments
        - o: the object to be compared for equality with this map

        Returns
        - `True` if the specified object is equal to this map
        """
        ...


    def hashCode(self) -> int:
        """
        Returns the hash code value for this map.  The hash code of a map is
        defined to be the sum of the hash codes of each entry in the map.
        """
        ...


    def clone(self) -> "EnumMap"["K", "V"]:
        """
        Returns a shallow copy of this enum map. The values themselves
        are not cloned.

        Returns
        - a shallow copy of this enum map
        """
        ...



from .WrappingScheduledExecutorService import WrappingScheduledExecutorService
from .WrappingExecutorService import WrappingExecutorService
from .Uninterruptibles import Uninterruptibles
from .UncheckedTimeoutException import UncheckedTimeoutException
from .UncheckedExecutionException import UncheckedExecutionException
from .UncaughtExceptionHandlers import UncaughtExceptionHandlers
from .TrustedListenableFutureTask import TrustedListenableFutureTask
from .TimeoutFuture import TimeoutFuture
from .TimeLimiter import TimeLimiter
from .ThreadFactoryBuilder import ThreadFactoryBuilder
from .Striped import Striped
from .SmoothRateLimiter import SmoothRateLimiter
from .SimpleTimeLimiter import SimpleTimeLimiter
from .SettableFuture import SettableFuture
from .ServiceManagerBridge import ServiceManagerBridge
from .ServiceManager import ServiceManager
from .Service import Service
from .SequentialExecutor import SequentialExecutor
from .Runnables import Runnables
from .RateLimiter import RateLimiter
from .Platform import Platform
from .Partially import Partially
from .ParametricNullness import ParametricNullness
from .OverflowAvoidingLockSupport import OverflowAvoidingLockSupport
from .NullnessCasts import NullnessCasts
from .MoreExecutors import MoreExecutors
from .Monitor import Monitor
from .ListeningScheduledExecutorService import ListeningScheduledExecutorService
from .ListeningExecutorService import ListeningExecutorService
from .ListenerCallQueue import ListenerCallQueue
from .ListenableScheduledFuture import ListenableScheduledFuture
from .ListenableFutureTask import ListenableFutureTask
from .ListenableFuture import ListenableFuture
from .LazyLogger import LazyLogger
from .JdkFutureAdapters import JdkFutureAdapters
from .InterruptibleTask import InterruptibleTask
from .Internal import Internal
from .ImmediateFuture import ImmediateFuture
from .GwtFuturesCatchingSpecialization import GwtFuturesCatchingSpecialization
from .GwtFluentFutureCatchingSpecialization import GwtFluentFutureCatchingSpecialization
from .FuturesGetChecked import FuturesGetChecked
from .Futures import Futures
from .FutureCallback import FutureCallback
from .ForwardingLock import ForwardingLock
from .ForwardingListeningExecutorService import ForwardingListeningExecutorService
from .ForwardingListenableFuture import ForwardingListenableFuture
from .ForwardingFuture import ForwardingFuture
from .ForwardingFluentFuture import ForwardingFluentFuture
from .ForwardingExecutorService import ForwardingExecutorService
from .ForwardingCondition import ForwardingCondition
from .ForwardingBlockingQueue import ForwardingBlockingQueue
from .ForwardingBlockingDeque import ForwardingBlockingDeque
from .FluentFuture import FluentFuture
from .FakeTimeLimiter import FakeTimeLimiter
from .ExecutionSequencer import ExecutionSequencer
from .ExecutionList import ExecutionList
from .ExecutionError import ExecutionError
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault
from .DirectExecutorService import DirectExecutorService
from .DirectExecutor import DirectExecutor
from .CycleDetectingLockFactory import CycleDetectingLockFactory
from .CombinedFuture import CombinedFuture
from .CollectionFuture import CollectionFuture
from .ClosingFuture import ClosingFuture
from .Callables import Callables
from .Atomics import Atomics
from .AtomicLongMap import AtomicLongMap
from .AtomicDoubleArray import AtomicDoubleArray
from .AtomicDouble import AtomicDouble
from .AsyncFunction import AsyncFunction
from .AsyncCallable import AsyncCallable
from .AggregateFutureState import AggregateFutureState
from .AggregateFuture import AggregateFuture
from .AbstractTransformFuture import AbstractTransformFuture
from .AbstractService import AbstractService
from .AbstractScheduledService import AbstractScheduledService
from .AbstractListeningExecutorService import AbstractListeningExecutorService
from .AbstractIdleService import AbstractIdleService
from .AbstractFuture import AbstractFuture
from .AbstractExecutionThreadService import AbstractExecutionThreadService
from .AbstractCatchingFuture import AbstractCatchingFuture


"""
Python module generated from Java source file org.bukkit.inventory.view.builder.MerchantInventoryViewBuilder

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Server
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import Merchant
from org.bukkit.inventory.view.builder import *
from typing import Any, Callable, Iterable, Tuple


class MerchantInventoryViewBuilder(InventoryViewBuilder):
    """
    An InventoryViewBuilder for creating merchant views
    
    Type `<V>`: the type of InventoryView created by this builder
    """

    def copy(self) -> "MerchantInventoryViewBuilder"["V"]:
        ...


    def title(self, title: str) -> "MerchantInventoryViewBuilder"["V"]:
        ...


    def merchant(self, merchant: "Merchant") -> "MerchantInventoryViewBuilder"["V"]:
        """
        Adds a merchant to this builder

        Arguments
        - merchant: the merchant

        Returns
        - this builder
        """
        ...


    def checkReachable(self, checkReachable: bool) -> "MerchantInventoryViewBuilder"["V"]:
        """
        Determines whether or not the server should check if the player can reach
        the location.
        
        Given checkReachable is provided and a virtual merchant is provided to
        the builder from Server.createMerchant(String) this method will
        have no effect on the actual menu status.

        Arguments
        - checkReachable: whether or not to check if the view is "reachable"

        Returns
        - this builder
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.BlockSupport

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockSupport(Enum):
    """
    Represents a level of support a block can give on one of its faces.
    
    Any given face on a block may support anywhere between none and all three of the
    values in this enum. The top face of a grass block for instance can support blocks
    that require a full, center, or rigid face. On the contrary, all sides except the
    bottom of a camp fire cannot support any blocks, while the bottom face can support
    blocks that require a full or center face (such as a ceiling button).
    """

    FULL = 0
    """
    The face is treated as a full block. For example, the side of a stair is
    <strong>not</strong> a full face and cannot support a wall torch, whereas the
    back and bottom of a stair are considered full.
    """
    CENTER = 1
    """
    The face is capable of supporting blocks towards the center. For example, a
    wall or a fence post can support a standing torch as there is a solid component
    in the middle of the block.
    """
    RIGID = 2
    """
    The face is capable of supporting fragile blocks such as rails. Most
    full-supportable top faces are rigid, unlike walls and posts, or the side of a
    stone block, none of which are rigid.
    """



"""
Python module generated from Java source file org.bukkit.event.player.PlayerChatEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit import Warning
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerChatEvent(PlayerEvent, Cancellable):
    """
    Holds information for player chat and commands

    Deprecated
    - This event will fire from the main thread and allows the use of
        all of the Bukkit API, unlike the AsyncPlayerChatEvent.
        
        Listening to this event forces chat to wait for the main thread which
        causes delays for chat. AsyncPlayerChatEvent is the encouraged
        alternative for thread safe implementations.
    """

    def __init__(self, player: "Player", message: str):
        ...


    def __init__(self, player: "Player", message: str, format: str, recipients: set["Player"]):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getMessage(self) -> str:
        """
        Gets the message that the player is attempting to send

        Returns
        - Message the player is attempting to send
        """
        ...


    def setMessage(self, message: str) -> None:
        """
        Sets the message that the player will send

        Arguments
        - message: New message that the player will send
        """
        ...


    def setPlayer(self, player: "Player") -> None:
        """
        Sets the player that this message will display as, or command will be
        executed as

        Arguments
        - player: New player which this event will execute as
        """
        ...


    def getFormat(self) -> str:
        """
        Gets the format to use to display this chat message

        Returns
        - String.Format compatible format string
        """
        ...


    def setFormat(self, format: str) -> None:
        """
        Sets the format to use to display this chat message

        Arguments
        - format: String.Format compatible format string
        """
        ...


    def getRecipients(self) -> set["Player"]:
        """
        Gets a set of recipients that this chat message will be displayed to

        Returns
        - All Players who will see this chat message
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerCommandSendEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerCommandSendEvent(PlayerEvent):
    """
    This event is called when the list of available server commands is sent to
    the player.
    
    Commands may be removed from display using this event, but implementations
    are not required to securely remove all traces of the command. If secure
    removal of commands is required, then the command should be assigned a
    permission which is not granted to the player.
    """

    def __init__(self, player: "Player", commands: Iterable[str]):
        ...


    def getCommands(self) -> Iterable[str]:
        """
        Returns a mutable collection of all top level commands to be sent.
        
        It is not legal to add entries to this collection, only remove them.
        Behaviour of adding entries is undefined.

        Returns
        - collection of all commands
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.inventory.Inventory

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import ListIterator
from org.bukkit import Location
from org.bukkit import Material
from org.bukkit.entity import HumanEntity
from org.bukkit.event.inventory import InventoryType
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class Inventory(Iterable):
    """
    Interface to the various inventories. Behavior relating to Material.AIR is unspecified.
    
    
    **Note that whilst .iterator() deals with the entire inventory, add
    / contains / remove methods deal only with the storage contents.**
    
    **Consider using .getContents() and .getStorageContents() for
    specific iteration.**

    See
    - .getStorageContents()
    """

    def getSize(self) -> int:
        """
        Returns the size of the inventory

        Returns
        - The size of the inventory
        """
        ...


    def getMaxStackSize(self) -> int:
        """
        Returns the maximum stack size for an ItemStack in this inventory.

        Returns
        - The maximum size for an ItemStack in this inventory.
        """
        ...


    def setMaxStackSize(self, size: int) -> None:
        """
        This method allows you to change the maximum stack size for an
        inventory.
        
        **Caveats:**
        
        - Not all inventories respect this value.
        - Stacks larger than 127 may be clipped when the world is saved.
        - This value is not guaranteed to be preserved; be sure to set it
            before every time you want to set a slot over the max stack size.
        - Stacks larger than the default max size for this type of inventory
            may not display correctly in the client.

        Arguments
        - size: The new maximum stack size for items in this inventory.
        """
        ...


    def getItem(self, index: int) -> "ItemStack":
        """
        Returns the ItemStack found in the slot at the given index

        Arguments
        - index: The index of the Slot's ItemStack to return

        Returns
        - The ItemStack in the slot
        """
        ...


    def setItem(self, index: int, item: "ItemStack") -> None:
        """
        Stores the ItemStack at the given index of the inventory.

        Arguments
        - index: The index where to put the ItemStack
        - item: The ItemStack to set
        """
        ...


    def addItem(self, *items: Tuple["ItemStack", ...]) -> dict["Integer", "ItemStack"]:
        """
        Stores the given ItemStacks in the inventory. This will try to fill
        existing stacks and empty slots as well as it can.
        
        The returned HashMap contains what it couldn't store, where the key is
        the index of the parameter, and the value is the ItemStack at that
        index of the varargs parameter. If all items are stored, it will return
        an empty HashMap.
        
        If you pass in ItemStacks which exceed the maximum stack size for the
        Material, first they will be added to partial stacks where
        Material.getMaxStackSize() is not exceeded, up to
        Material.getMaxStackSize(). When there are no partial stacks left
        stacks will be split on Inventory.getMaxStackSize() allowing you to
        exceed the maximum stack size for that material.
        
        It is known that in some implementations this method will also set
        the inputted argument amount to the number of that item not placed in
        slots.

        Arguments
        - items: The ItemStacks to add

        Returns
        - A HashMap containing items that didn't fit.

        Raises
        - IllegalArgumentException: if items or any element in it is null
        """
        ...


    def removeItem(self, *items: Tuple["ItemStack", ...]) -> dict["Integer", "ItemStack"]:
        """
        Removes the given ItemStacks from the inventory.
        
        It will try to remove 'as much as possible' from the types and amounts
        you give as arguments.
        
        The returned HashMap contains what it couldn't remove, where the key is
        the index of the parameter, and the value is the ItemStack at that
        index of the varargs parameter. If all the given ItemStacks are
        removed, it will return an empty HashMap.
        
        It is known that in some implementations this method will also set the
        inputted argument amount to the number of that item not removed from
        slots.

        Arguments
        - items: The ItemStacks to remove

        Returns
        - A HashMap containing items that couldn't be removed.

        Raises
        - IllegalArgumentException: if items is null
        """
        ...


    def getContents(self) -> list["ItemStack"]:
        """
        Returns all ItemStacks from the inventory

        Returns
        - An array of ItemStacks from the inventory. Individual items may be null.
        """
        ...


    def setContents(self, items: list["ItemStack"]) -> None:
        """
        Completely replaces the inventory's contents. Removes all existing
        contents and replaces it with the ItemStacks given in the array.

        Arguments
        - items: A complete replacement for the contents; the length must
            be less than or equal to .getSize().

        Raises
        - IllegalArgumentException: If the array has more items than the
            inventory.
        """
        ...


    def getStorageContents(self) -> list["ItemStack"]:
        """
        Return the contents from the section of the inventory where items can
        reasonably be expected to be stored. In most cases this will represent
        the entire inventory, but in some cases it may exclude armor or result
        slots.
        
        It is these contents which will be used for add / contains / remove
        methods which look for a specific stack.

        Returns
        - inventory storage contents. Individual items may be null.
        """
        ...


    def setStorageContents(self, items: list["ItemStack"]) -> None:
        """
        Put the given ItemStacks into the storage slots

        Arguments
        - items: The ItemStacks to use as storage contents

        Raises
        - IllegalArgumentException: If the array has more items than the
        inventory.
        """
        ...


    def contains(self, material: "Material") -> bool:
        """
        Checks if the inventory contains any ItemStacks with the given
        material.

        Arguments
        - material: The material to check for

        Returns
        - True if an ItemStack is found with the given Material

        Raises
        - IllegalArgumentException: if material is null
        """
        ...


    def contains(self, item: "ItemStack") -> bool:
        """
        Checks if the inventory contains any ItemStacks matching the given
        ItemStack.
        
        This will only return True if both the type and the amount of the stack
        match.

        Arguments
        - item: The ItemStack to match against

        Returns
        - False if item is null, True if any exactly matching ItemStacks
            were found
        """
        ...


    def contains(self, material: "Material", amount: int) -> bool:
        """
        Checks if the inventory contains any ItemStacks with the given
        material, adding to at least the minimum amount specified.

        Arguments
        - material: The material to check for
        - amount: The minimum amount

        Returns
        - True if amount is less than 1, True if enough ItemStacks were
            found to add to the given amount

        Raises
        - IllegalArgumentException: if material is null
        """
        ...


    def contains(self, item: "ItemStack", amount: int) -> bool:
        """
        Checks if the inventory contains at least the minimum amount specified
        of exactly matching ItemStacks.
        
        An ItemStack only counts if both the type and the amount of the stack
        match.

        Arguments
        - item: the ItemStack to match against
        - amount: how many identical stacks to check for

        Returns
        - False if item is null, True if amount less than 1, True if
            amount of exactly matching ItemStacks were found

        See
        - .containsAtLeast(ItemStack, int)
        """
        ...


    def containsAtLeast(self, item: "ItemStack", amount: int) -> bool:
        """
        Checks if the inventory contains ItemStacks matching the given
        ItemStack whose amounts sum to at least the minimum amount specified.

        Arguments
        - item: the ItemStack to match against
        - amount: the minimum amount

        Returns
        - False if item is null, True if amount less than 1, True if
            enough ItemStacks were found to add to the given amount
        """
        ...


    def all(self, material: "Material") -> dict["Integer", "ItemStack"]:
        """
        Returns a HashMap with all slots and ItemStacks in the inventory with
        the given Material.
        
        The HashMap contains entries where, the key is the slot index, and the
        value is the ItemStack in that slot. If no matching ItemStack with the
        given Material is found, an empty map is returned.

        Arguments
        - material: The material to look for

        Returns
        - A HashMap containing the slot index, ItemStack pairs

        Raises
        - IllegalArgumentException: if material is null
        """
        ...


    def all(self, item: "ItemStack") -> dict["Integer", "ItemStack"]:
        """
        Finds all slots in the inventory containing any ItemStacks with the
        given ItemStack. This will only match slots if both the type and the
        amount of the stack match
        
        The HashMap contains entries where, the key is the slot index, and the
        value is the ItemStack in that slot. If no matching ItemStack with the
        given Material is found, an empty map is returned.

        Arguments
        - item: The ItemStack to match against

        Returns
        - A map from slot indexes to item at index
        """
        ...


    def first(self, material: "Material") -> int:
        """
        Finds the first slot in the inventory containing an ItemStack with the
        given material

        Arguments
        - material: The material to look for

        Returns
        - The slot index of the given Material or -1 if not found

        Raises
        - IllegalArgumentException: if material is null
        """
        ...


    def first(self, item: "ItemStack") -> int:
        """
        Returns the first slot in the inventory containing an ItemStack with
        the given stack. This will only match a slot if both the type and the
        amount of the stack match

        Arguments
        - item: The ItemStack to match against

        Returns
        - The slot index of the given ItemStack or -1 if not found
        """
        ...


    def firstEmpty(self) -> int:
        """
        Returns the first empty Slot.

        Returns
        - The first empty Slot found, or -1 if no empty slots.
        """
        ...


    def isEmpty(self) -> bool:
        """
        Check whether or not this inventory is empty. An inventory is considered
        to be empty if there are no ItemStacks in any slot of this inventory.

        Returns
        - True if empty, False otherwise
        """
        ...


    def remove(self, material: "Material") -> None:
        """
        Removes all stacks in the inventory matching the given material.

        Arguments
        - material: The material to remove

        Raises
        - IllegalArgumentException: if material is null
        """
        ...


    def remove(self, item: "ItemStack") -> None:
        """
        Removes all stacks in the inventory matching the given stack.
        
        This will only match a slot if both the type and the amount of the
        stack match

        Arguments
        - item: The ItemStack to match against
        """
        ...


    def clear(self, index: int) -> None:
        """
        Clears out a particular slot in the index.

        Arguments
        - index: The index to empty.
        """
        ...


    def clear(self) -> None:
        """
        Clears out the whole Inventory.
        """
        ...


    def getViewers(self) -> list["HumanEntity"]:
        """
        Gets a list of players viewing the inventory. Note that a player is
        considered to be viewing their own inventory and internal crafting
        screen even when said inventory is not open. They will normally be
        considered to be viewing their inventory even when they have a
        different inventory screen open, but it's possible for customized
        inventory screens to exclude the viewer's inventory, so this should
        never be assumed to be non-empty.

        Returns
        - A list of HumanEntities who are viewing this Inventory.
        """
        ...


    def getType(self) -> "InventoryType":
        """
        Returns what type of inventory this is.

        Returns
        - The InventoryType representing the type of inventory.
        """
        ...


    def getHolder(self) -> "InventoryHolder":
        """
        Gets the block or entity belonging to the open inventory

        Returns
        - The holder of the inventory; null if it has no holder.
        """
        ...


    def iterator(self) -> "ListIterator"["ItemStack"]:
        ...


    def iterator(self, index: int) -> "ListIterator"["ItemStack"]:
        """
        Returns an iterator starting at the given index. If the index is
        positive, then the first call to next() will return the item at that
        index; if it is negative, the first call to previous will return the
        item at index (getSize() + index).

        Arguments
        - index: The index.

        Returns
        - An iterator.
        """
        ...


    def getLocation(self) -> "Location":
        """
        Get the location of the block or entity which corresponds to this inventory. May return null if this container
        was custom created or is a virtual / subcontainer.

        Returns
        - location or null if not applicable.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.EmptyImmutableListMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from typing import Any, Callable, Iterable, Tuple


class EmptyImmutableListMultimap(ImmutableListMultimap):
    """
    Implementation of ImmutableListMultimap with no entries.

    Author(s)
    - Jared Levy
    """

    def asMap(self) -> "ImmutableMap"["Object", Iterable["Object"]]:
        ...



"""
Python module generated from Java source file org.bukkit.help.GenericCommandHelpTopic

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Joiner
from org.bukkit import ChatColor
from org.bukkit.command import Command
from org.bukkit.command import CommandSender
from org.bukkit.command import ConsoleCommandSender
from org.bukkit.help import *
from typing import Any, Callable, Iterable, Tuple


class GenericCommandHelpTopic(HelpTopic):
    """
    Lacking an alternative, the help system will create instances of
    GenericCommandHelpTopic for each command in the server's CommandMap. You
    can use this class as a base class for custom help topics, or as an example
    for how to write your own.
    """

    def __init__(self, command: "Command"):
        ...


    def canSee(self, sender: "CommandSender") -> bool:
        ...



"""
Python module generated from Java source file java.io.IOException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class IOException(Exception):
    """
    Signals that an I/O exception of some sort has occurred. This
    class is the general class of exceptions produced by failed or
    interrupted I/O operations.

    See
    - java.io.OutputStream

    Since
    - 1.0
    """

    def __init__(self):
        """
        Constructs an `IOException` with `null`
        as its error detail message.
        """
        ...


    def __init__(self, message: str):
        """
        Constructs an `IOException` with the specified detail message.

        Arguments
        - message: The detail message (which is saved for later retrieval
               by the .getMessage() method)
        """
        ...


    def __init__(self, message: str, cause: "Throwable"):
        """
        Constructs an `IOException` with the specified detail message
        and cause.
        
         Note that the detail message associated with `cause` is
        *not* automatically incorporated into this exception's detail
        message.

        Arguments
        - message: The detail message (which is saved for later retrieval
               by the .getMessage() method)
        - cause: The cause (which is saved for later retrieval by the
               .getCause() method).  (A null value is permitted,
               and indicates that the cause is nonexistent or unknown.)

        Since
        - 1.6
        """
        ...


    def __init__(self, cause: "Throwable"):
        """
        Constructs an `IOException` with the specified cause and a
        detail message of `(cause==null ? null : cause.toString())`
        (which typically contains the class and detail message of `cause`).
        This constructor is useful for IO exceptions that are little more
        than wrappers for other throwables.

        Arguments
        - cause: The cause (which is saved for later retrieval by the
               .getCause() method).  (A null value is permitted,
               and indicates that the cause is nonexistent or unknown.)

        Since
        - 1.6
        """
        ...
